// Copyright 2017 by Yun Xiang, Tam Chantem, and Robert Dick.
// All rights reserved.

/*Copyright 2010, Yun Xiang<xiangyun@umich.edu>.*/
#ifndef PARSE_H_
#define PARSE_H_
#include "RVector.h"
#include "RStd.h"
#include "Cont.h"
#include "Mttf.h"

#include <cstddef>
#include <string>
#include <exception>
#include <fstream>
#include <vector>
#include "FVector.h"
#include "RStd.h"
#include "RString.h"
#include "configuration.h"

using namespace std;
using namespace rstd;

//namespace rstd {
//template <typename> class RVector;
//class RGen;
//}

class Cycle;

class GLOBAL {
 public:
          GLOBAL() { }
	  RVector<string> type;
	  RVector<double> em;
	  RVector<double> sm;
	  RVector<double> tc;
	  RVector<double> tddb;
	  double t;
};

class PE {
public:
	virtual ~PE() {}

	void set_values(double ref_em_a, double ref_tc_a, 
			double ref_sm_a, double ref_tddb_a, 
			double ref_vol_a, double ref_tmp_a, 
			double ref_feq_a, int device_num_a,
			double em_age_a, double tc_age_a,
			double sm_age_a, double tddb_age_a);
	double mu;
	double sigma;
	double age;
	double em_age;
	double tc_age;
	double sm_age;
	double tddb_age;
	double em_increase;
	double tc_increase;
	double sm_increase;
	double tddb_increase;
	double interval;
	double em_sigma;
	double tc_sigma;
	double sm_sigma;
	double tddb_sigma;
     	RVector<double> em_mu;
	RVector<double> tc_mu;
	RVector<double> sm_mu;
	RVector<double> tddb_mu;
	double total_em;
	double total_sm;
	double total_tc;
	double total_tddb;

	RVector<double> temp;
	RVector<double> time;
	RVector<double> vol;
	RVector<double> freq;

	RVector<double> peak;
	RVector<double> valley;
	RVector<Cycle> half_cycles;
	RVector<Cycle> full_cycles;
	RVector<Cycle> cycles;
	int device_num;
//	int TC_flag; // it should be 0 during normal operation!

	double ref_em() const { return ref_em_; }
	double ref_tc() const { return ref_tc_; }
	double ref_sm() const { return ref_sm_; }
	double ref_tddb() const { return ref_tddb_; }

	double ref_vol() const { return ref_vol_; }
	double ref_tmp() const { return ref_tmp_; }
	double ref_feq() const { return ref_feq_; }
	void em_mttf(const RVector<double> &v_scale, const RVector<double> &t_scale, const RVector<double> &f_scale);
	void tc_mttf(const RVector<double> & tmp);
	void tc_mttf();
	void sm_mttf(const RVector<double> & t_scale);
	void tddb_mttf(const RVector<double> & v_scale, const RVector<double> & t_scale);
	
	void mttf(const RVector<double> & v_scale, const RVector<double> & t_scale, const RVector<double> & f_scale);

	void aging (double t);
	void age_update();
	void age_update(double time_limit);
	void flow_profile();
  void flow_profile(RVector<double> &local_temp); 
	void RFC();
	void RFC(const RVector<double> & local_temp) ;
	void reload(const RVector<double> & rtime, const RVector<double> & rtemp, const RVector<double> & rvol, const RVector<double> & rfreq, const RVector<Cycle> & rcycle);

	/*###################################################################*/

//	PE(double price_a) : price_(price_a) {}
	PE( 	   
		double ref_em_a = REF_EM_MTTF, 
		double ref_tc_a = REF_TC_MTTF, 
		double ref_sm_a = REF_SM_MTTF, 
		double ref_tddb_a = REF_TDDB_MTTF, 
		double ref_vol_a = REF_VOLTAGE, 
		double ref_tmp_a = REF_TEMPERATURE, 
		double ref_feq_a = REF_FREQUENCE
		//double vol_a = REF_VOLTAGE,
		//double tmp_a = REF_TEMPERATURE,
		//double freq_a = REF_FREQUENCE,
		  ) : 
		  mu(10.0),
		  sigma(5.0),
		  //vol_(vol_a),
		  //tmp_(tmp_a),
		  //freq_(freq_a),
		  age(0),
		  em_age(0),
		  tc_age(0),
		  sm_age(0),
		  tddb_age(0),
		  em_increase(0),
		  tc_increase(0),
		  sm_increase(0),
		  tddb_increase(0),
		    total_em(0),
		    total_sm(0),
		    total_tc(0),
		    total_tddb(0),
		ref_em_(ref_em_a), 
		ref_tc_(ref_tc_a), 
		ref_sm_(ref_sm_a), 
		ref_tddb_(ref_tddb_a), 
		ref_vol_(ref_vol_a), 
		ref_tmp_(ref_tmp_a), 
		    ref_feq_(ref_feq_a)
		  
		{
//			ref_par_mttf_ = pf_mttf(ref_tmp_, 0, PE::PARFAIL);
//			ref_whole_mttf_ = pf_mttf(ref_tmp_, 0, PE::WFAIL);
		}

 protected:

	double ref_em_;		// reference point for the mean time to failure	
	double ref_tc_;
	double ref_sm_;
	double ref_tddb_;


	double ref_vol_;	// reference point of voltage for the MTTF
	double ref_tmp_;	// reference point of temperature for the MTTF
	double ref_feq_;	// reference point of frequence for the MTTF 
	
	RVector<double> vol_;
	double tmp_;
	RVector<double> freq_;
	
	RVector<double> temperature_;
	RVector<double> time_;
};

void parse(RVector<PE> & pe, int argc, const char *argv, RVector<string> survive_list);
int parse(RVector<PE> & pe, const char *rel_file, RVector<string> &survive_list, RVector<RVector<RVector<double> > > &therm_profile, RVector<RVector<RVector<double> > > &time_profile, RVector<RVector<RVector<double> > > &vol_profile, RVector<RVector<RVector<double> > > &freq_profile, RVector<RVector<RVector<Cycle> > > &cycle_profile, GLOBAL &def);

//void convert_grids_to_flrplan (std::vector<Element_type> *thermal_profile, int profile_size, const char *flr_input);
extern double TIME_STEPS;
#endif
